@@ -1,5 +1,5 @@
 
-/* $Id: os_matrix.h,v 1.4 2003/08/20 05:30:16 mederchik Exp $ */
+/* $Id: os_matrix.h,v 1.5 2005/02/08 20:00:35 mederchik Exp $ */
 /*
 ** Copyright (C) 2001 Fyodor Yarochkin <fygrave@tigerteam.net>,
 **                    Ofir Arkin       <ofir@sys-security.com>
@@ -81,7 +81,7 @@ class OS_Matrix {
     public:
         OS_Matrix(int);
         virtual ~OS_Matrix(void);
-        void add_result(int, int, int);
+        void add_result(int, int, int, int times = 1);
         /* returns top num scored OS id */
         int get_top(int);
         /* returns given os_id score */
