@@ -1,4 +1,4 @@
-/* $Id: target.cc,v 1.13 2003/09/24 11:57:04 mederchik Exp $ */
+/* $Id: target.cc,v 1.15 2005/02/08 20:00:35 mederchik Exp $ */
 /*
 ** Copyright (C) 2001 Fyodor Yarochkin <fygrave@tigerteam.net>,
 **                    Ofir Arkin       <ofir@sys-security.com>
@@ -115,7 +115,16 @@ unsigned long Target_Net::getnext(void) {
 
     if (counter > ntohl(addr|mask^0xffffffff))
         return 0;
-    return (htonl(counter++));
+	if (counter == htonl(addr)) { // network number
+		if (mask != 0xffffffff) { // special case of /32
+			counter++;
+		}
+		return (htonl(counter++));
+	}
+	if (ntohl(mask^0xffffffff) == (counter & ntohl(mask^0xffffffff))){ // broadcast address
+		return 0;
+	}
+	return (htonl(counter++));
 }
 
 
@@ -250,7 +259,7 @@ int Target::os_probe(void) {
 		ui->log("%s", (fingerprint.get_sig(&keywordcount)).c_str());
 		if (keywordcount < xmh->loaded_mods_num(XPROBE_MODULE_OSTEST)) {
 			ui->msg("[+] GENERATED FINGERPRINT IS INCOMPLETE!\n");
-			ui->msg("[+] Please make sure you target is not firewalled and you have specified at least one open TCP port and one closed UDP port!\n");
+			ui->msg("[+] Please make sure you target is not firewalled and you have specified at least one open TCP port and one closed TCP and UDP ports!\n");
 		}
 	} else {
 		xml->log(XPROBELOG_GUESS_SESS_START, "OS guess");
