@@ -83,7 +83,7 @@ int TTL_Mod::get_ttl_distance(Target *tgt) {
 	if ((double)tgt->get_rtt() == 0.0 || (tgt->get_port(IPPROTO_TCP, XPROBE_TARGETP_OPEN) == -1 &&
 			 	      tgt->get_port(IPPROTO_TCP, XPROBE_TARGETP_CLOSED) == -1 &&
 			 	      tgt->get_port(IPPROTO_UDP, XPROBE_TARGETP_CLOSED) == -1)) {
-		ui->msg("[+] No distance calculation. %s appears to be dead or no ports known\n",
+		ui->msg("[-] No distance calculation. %s appears to be dead or no ports known\n",
 				inet_ntoa(tgt->get_addr()));
 		return FAIL;
 	}
