@@ -1,4 +1,4 @@
-/* $Id: icmp_addrmask.cc,v 1.12 2003/12/29 12:21:59 mederchik Exp $ */
+/* $Id: icmp_addrmask.cc,v 1.13 2005/02/08 20:00:37 mederchik Exp $ */
 /*
 ** Copyright (C) 2001 Fyodor Yarochkin <fygrave@tigerteam.net>,
 **                    Ofir Arkin       <ofir@sys-security.com>
@@ -94,10 +94,9 @@ int ICMP_Addrmask_Mod::fini(void) {
 
 int ICMP_Addrmask_Mod::parse_keyword(int os_id, const char *kwd, const char *val) {
 
-	for (s_i=kwd_chk.begin(); s_i != kwd_chk.end(); s_i++)
-		if (s_i->first == kwd)
+	if ((s_i = kwd_chk.find(kwd)) != kwd_chk.end()) {
 			return s_i->second->parse_param(os_id, val);
-
+	}
 	ui->msg ("No keywords matched(%s=%s)!", kwd, val);
 	return FAIL;
 }
