@@ -1,4 +1,4 @@
-/* $Id: icmp_echo_id.cc,v 1.14 2003/12/29 12:22:00 mederchik Exp $ */
+/* $Id: icmp_echo_id.cc,v 1.15 2005/02/08 20:00:51 mederchik Exp $ */
 /*
 ** Copyright (C) 2001 Fyodor Yarochkin <fygrave@tigerteam.net>,
 **                    Ofir Arkin       <ofir@sys-security.com>
@@ -45,6 +45,7 @@ int icmp_echo_id_mod_init(Xprobe_Module_Hdlr *pt, char *nm) {
     module->set_name(nm);
     xprobe_mdebug(XPROBE_DEBUG_MODULES, "Initializing the ICMP ECHO ID module\n");
     pt->register_module(module);
+    pt->add_keyword(module->get_id(),"icmp_echo_reply");
     pt->add_keyword(module->get_id(),"icmp_echo_code");
     pt->add_keyword(module->get_id(),"icmp_echo_ip_id");
     pt->add_keyword(module->get_id(),"icmp_echo_tos_bits");
@@ -61,7 +62,9 @@ ICMP_Echo_Id_Mod::ICMP_Echo_Id_Mod(void): Xprobe_Module(XPROBE_MODULE_OSTEST, "f
     ICMP_Echo_Tos_Chk  *ietc = new ICMP_Echo_Tos_Chk;
     ICMP_Echo_Df_Bit_Chk *iedbc = new ICMP_Echo_Df_Bit_Chk;
     ICMP_Echo_Reply_Ttl_Chk *iertc = new ICMP_Echo_Reply_Ttl_Chk;
+    ICMP_Echo_Reply_Check *ierc= new ICMP_Echo_Reply_Check;
 
+    kwd_chk.insert(pair<string, Xprobe_Module_Param_ICMP *>("icmp_echo_reply", ierc));
     kwd_chk.insert(pair<string, Xprobe_Module_Param_ICMP *>("icmp_echo_code", iecc));
     kwd_chk.insert(pair<string, Xprobe_Module_Param_ICMP *>("icmp_echo_ip_id", ieic));
     kwd_chk.insert(pair<string, Xprobe_Module_Param_ICMP *>("icmp_echo_tos_bits", ietc));
@@ -130,16 +133,13 @@ void ICMP_Echo_Id_Mod::sig_ttl_insert(int os_id, int val) {
 
 
 int ICMP_Echo_Id_Mod::parse_keyword(int os_id, const char *kwd, const char *val)  {
-         map <string, Xprobe_Module_Param_ICMP *>::iterator s_i;
+	map <string, Xprobe_Module_Param_ICMP *>::iterator s_i;
 
-    xprobe_debug(XPROBE_DEBUG_MODULES, "Parsing for %i : %s  = %s\n",
+    xprobe_debug(XPROBE_DEBUG_SIGNATURES, "Parsing for %i : %s  = %s\n",
                                                         os_id,  kwd, val);
-/* go through the list of keywords */
-    for (s_i = kwd_chk.begin(); s_i != kwd_chk.end(); s_i++) 
-        if ((*s_i).first == kwd) {
+	if ((s_i=kwd_chk.find(kwd)) != kwd_chk.end()) {
             return((*s_i).second->parse_param(os_id, val));
-        }
-       
+	}
     ui->msg("Ooops..none matched %s %s\n", kwd, val);   
     return FAIL;
 
@@ -223,6 +223,7 @@ void ICMP_Echo_Id_Mod::generate_signature(Target *tg, ICMP *pack, ICMP *orig) {
 #       icmp_echo_reply_ttl = [>< decimal num]
 */
 	if (!pack->timeout()) {
+		tg->signature("icmp_echo_reply", "y");
 		keyword = "icmp_echo_code";
 		if (pack->get_code() == 0)
 			value="0";
@@ -263,6 +264,7 @@ void ICMP_Echo_Id_Mod::generate_signature(Target *tg, ICMP *pack, ICMP *orig) {
 		tg->signature(keyword, value);
 	} else {
 		tg->signature("# No ICMP Echo reply received", "");
+		tg->signature("icmp_echo_reply", "n");
 		tg->signature("icmp_echo_code", "");
 		tg->signature("icmp_echo_ip_id", "");
 		tg->signature("icmp_echo_tos_bits","");
@@ -306,3 +308,13 @@ int ICMP_Echo_Reply_Ttl_Chk::check_param(ICMP *ip_pkt, ICMP *orig_pkt, OS_Matrix
    return (add_param(ip_pkt->get_ttl(), orig_pkt->get_ttl(), os));
 }
 
+int ICMP_Echo_Reply_Check::check_param(ICMP *ip_pkt, ICMP *orig_pkt, OS_Matrix *os) {
+	int gotp=ip_pkt->timeout() ? 0 : 1;
+	// suspend warning
+	orig_pkt->timeout();
+	add_param(gotp, 0, os);
+	if (!gotp) {
+		gen_match(5, os);
+	}
+	return OK;
+}
