@@ -1,4 +1,4 @@
-/* $Id: icmp_inforeq.cc,v 1.11 2003/12/29 12:22:00 mederchik Exp $ */
+/* $Id: icmp_inforeq.cc,v 1.13 2005/02/09 18:36:46 mederchik Exp $ */
 /*
 ** Copyright (C) 2001 Fyodor Yarochkin <fygrave@tigerteam.net>,
 **                    Ofir Arkin       <ofir@sys-security.com>
@@ -60,7 +60,6 @@ ICMP_Inforeq_Mod::ICMP_Inforeq_Mod(void):Xprobe_Module(XPROBE_MODULE_OSTEST, "fi
 	kwd_chk.insert(pair<string, Xprobe_Module_Param_ICMP *>("icmp_info_reply",inforep));
 	kwd_chk.insert(pair<string, Xprobe_Module_Param_ICMP *>("icmp_info_reply_ttl",infottl));
 	kwd_chk.insert(pair<string, Xprobe_Module_Param_ICMP *>("icmp_info_reply_ip_id",infoid));
-	 
 }
 
 ICMP_Inforeq_Mod::~ICMP_Inforeq_Mod(void) {
@@ -96,12 +95,11 @@ int ICMP_Inforeq_Mod::fini(void) {
 
 int ICMP_Inforeq_Mod::parse_keyword(int os_id, const char *kwd, const char *val) {
 
-    xprobe_debug(XPROBE_DEBUG_MODULES, "Parsing for %i : %s  = %s\n",
+    xprobe_debug(XPROBE_DEBUG_SIGNATURES, "Parsing for %i : %s  = %s\n",
                                                         os_id,  kwd, val);
-
-	for (s_i = kwd_chk.begin(); s_i != kwd_chk.end(); s_i++) 
-		if (s_i->first == kwd)
-			return s_i->second->parse_param(os_id, val);
+	if ((s_i=kwd_chk.find(kwd)) != kwd_chk.end()) {
+		return s_i->second->parse_param(os_id, val);
+	}
 	return OK;
 }
 
