@@ -1,4 +1,4 @@
-/* $Id: icmp_timestamp.cc,v 1.11 2003/12/29 12:22:00 mederchik Exp $ */
+/* $Id: icmp_timestamp.cc,v 1.12 2005/02/08 20:01:03 mederchik Exp $ */
 /*
 ** Copyright (C) 2001 Fyodor Yarochkin <fygrave@tigerteam.net>,
 **                    Ofir Arkin       <ofir@sys-security.com>
@@ -93,10 +93,11 @@ int ICMP_Timestamp_Mod::fini(void) {
 
 int ICMP_Timestamp_Mod::parse_keyword(int os_id, const char *kwd, const char *val) {
 
-	for (s_i=kwd_chk.begin(); s_i != kwd_chk.end(); s_i++)
-		if (s_i->first == kwd)
-			return s_i->second->parse_param(os_id, val);
-	return OK;
+	if ((s_i=kwd_chk.find(kwd)) != kwd_chk.end()) {
+		return s_i->second->parse_param(os_id, val);
+	}
+	ui->msg("Ooops..none matched %s %s\n", kwd, val);
+	return FAIL;
 }
 
 
