@@ -54,6 +54,7 @@ TCP_Handshake_Mod::TCP_Handshake_Mod(void): Xprobe_Module(XPROBE_MODULE_OSTEST,
 	memset(opt_order, 0, sizeof(opt_order));
 	wscale = -1;
 	got_timestamp = false;
+	tse_first = tsv_first = tse_second = tsv_second = used_port = 0;
 
 }
 
@@ -72,13 +73,21 @@ int TCP_Handshake_Mod::init(void) {
 
 
 int TCP_Handshake_Mod::exec(Target *tg, OS_Matrix *os) {
+
+//	if (run_probe(tg, os) != FAIL) {
+		return run_probe(tg, os);
+//	}	
+//	return FAIL;
+}
+
+int TCP_Handshake_Mod::run_probe(Target *tg, OS_Matrix *os) {
 	map<string, Xprobe_Module_Param_TCP *>::iterator m_i;
 	TCP *request = new TCP(inet_ntoa(tg->get_addr()));
-	char tcp_options[40], buf[1024];
-	int done=0, ret, len, have_more_ports=1;
+	char buf[1024];
+	int done=0, ret, have_more_ports=1;
 	bool no_open_port;
-	unsigned int tse, tsv, ix=0;
-	unsigned short tcp_brute_ports[] = {80, 21, 25, 22, 139};
+	unsigned int ix=0;
+	unsigned short tcp_brute_ports[] = {80, 443, 23, 21, 25, 22, 139, 445, 6000};
 	Xprobe::Timeval tv;
 	struct in_addr local=tg->get_interface_addr(), remote=tg->get_addr();
 	TCP sn(inet_ntoa(local));
@@ -86,6 +95,13 @@ int TCP_Handshake_Mod::exec(Target *tg, OS_Matrix *os) {
 	tv = (double)tg->get_rtt()*50;
 	tv.tv_sec = 2;
 
+	if (used_port != 0 && (tse_first == 0 || tsv_first == 0)) {
+		// timestamp option was not received in previous request
+		xprobe_debug(XPROBE_DEBUG_MODULES, "--%s No timestamp recevied in previous TCP reply\n", get_name());
+		return OK;
+	}
+	if (used_port != 0)
+		usleep(10000);
 	/*
 	 * logic is as follows: 
 	 * - if user supplied the port with -p switch we'll go ahead and do a run
@@ -140,52 +156,81 @@ int TCP_Handshake_Mod::exec(Target *tg, OS_Matrix *os) {
 		} /* sniffpack loop */
 		done = 0;
 		if (!sn.timeout() && sn.get_flags() == (TH_SYN|TH_ACK)){
+				/*
+				 * add port to the list of the open ports on target
+				 */
+				tg->add_port(IPPROTO_TCP, request->get_dstport(), XPROBE_TARGETP_OPEN);
 				have_more_ports=false; // we got reply, no further packets required
 				xprobe_debug(XPROBE_DEBUG_MODULES, "[%s] Got SYN ACK packet.\n", get_name());
-				memset(tcp_options, 0, sizeof(tcp_options));
-				len = sn.get_tcpopt(tcp_options)-20;
-				parse_options(tcp_options, len);
-				for (m_i = kwd_chk.begin(); m_i != kwd_chk.end(); m_i++)
-					m_i->second->check_param(&sn, request, os);
-				for (w_i = wscale_map.begin(); w_i != wscale_map.end(); w_i++)
-					if (w_i->second == wscale) {
-						os->add_result(get_id(), w_i->first, XPROBE_MATCH_YES);
-					} else
-						os->add_result(get_id(), w_i->first, XPROBE_MATCH_NO);
-				for (o_i = options_map.begin(); o_i != options_map.end(); o_i++)
-					if (o_i->second == opt_order) {
-						os->add_result(get_id(), o_i->first, XPROBE_MATCH_YES);
-					} else 
-						os->add_result(get_id(), o_i->first, XPROBE_MATCH_NO);
-				if (got_timestamp) {
-					if (timestamps[0] == 0)
-						tsv = 0;
-					else
-						tsv = 1;
-
-					if (timestamps[1] == 0)
-						tse = 0;
-					else 
-						tse = 1;
-					for (ts_i = tsval.begin(); ts_i != tsval.end(); ts_i++)
-						if (ts_i->second == tsv)
-							os->add_result(get_id(), ts_i->first, XPROBE_MATCH_YES);
-					for (ts_i = tsecr.begin(); ts_i != tsecr.end(); ts_i++)
-						if (ts_i->second == tse)
-							os->add_result(get_id(), ts_i->first, XPROBE_MATCH_YES);
-				} else {
-					for (ts_i = tsval.begin(); ts_i != tsval.end(); ts_i++)  
-						if (ts_i->second == 2) /* NONE */
-							os->add_result(get_id(), ts_i->first, XPROBE_MATCH_YES);
-					for (ts_i = tsecr.begin(); ts_i != tsecr.end(); ts_i++)
-						if (ts_i->second == 2) /* NONE */
-							os->add_result(get_id(), ts_i->first, XPROBE_MATCH_YES);
+				//memset(tcp_options, 0, sizeof(tcp_options));
+				//len = sn.get_tcpopt(tcp_options)-20;
+				//parse_options(tcp_options, len);
+				memset(opt_order, 0, sizeof(opt_order));
+				sn.get_parsed_tcpopt(opt_order, sizeof(opt_order)-1);
+				xprobe_debug(XPROBE_DEBUG_MODULES, "[%s] GOT OPTIONS: %s\n", get_name(), opt_order);
+				if (strchr(opt_order, 'T')) {
+					got_timestamp=true;
+					timestamps[0] = sn.get_tcpopt_tsv();
+					timestamps[1] = sn.get_tcpopt_tse();
 				}
-				if (tg->generate_sig())
-					generate_signature(tg, &sn, request);
-			} else {
-				xprobe_debug(XPROBE_DEBUG_MODULES, "[%s] Did not receive SYN ACK packet. %d\n", get_name(), sn.get_dstport());
+				if (strchr(opt_order, 'W')) {
+					wscale = sn.get_wscale();
+				}
+				if (used_port != 0) { //second execution
+					xprobe_debug(XPROBE_DEBUG_MODULES, "TSV difference: %u\n", timestamps[0] - tsv_first);
+					xprobe_debug(XPROBE_DEBUG_MODULES, "TSE difference: %u\n", timestamps[1] - tse_first);
+				} else {
+					used_port = request->get_dstport();
+					for (m_i = kwd_chk.begin(); m_i != kwd_chk.end(); m_i++)
+						m_i->second->check_param(&sn, request, os);
+					for (w_i = wscale_map.begin(); w_i != wscale_map.end(); w_i++)
+						if (w_i->second == wscale) {
+							os->add_result(get_id(), w_i->first, XPROBE_MATCH_YES);
+						} else
+							os->add_result(get_id(), w_i->first, XPROBE_MATCH_NO);
+					for (o_i = options_map.begin(); o_i != options_map.end(); o_i++)
+						if (o_i->second == opt_order) {
+							os->add_result(get_id(), o_i->first, XPROBE_MATCH_YES);
+						} else 
+							os->add_result(get_id(), o_i->first, XPROBE_MATCH_NO);
+					if (got_timestamp) {
+						if (timestamps[0] == 0)
+							tsv_first = 0;
+						else
+							tsv_first = 1;
+
+						if (timestamps[1] == 0)
+							tse_first = 0;
+						else 
+							tse_first = 1;
+						for (ts_i = tsval.begin(); ts_i != tsval.end(); ts_i++)
+							if (ts_i->second == tsv_first)
+								os->add_result(get_id(), ts_i->first, XPROBE_MATCH_YES);
+						for (ts_i = tsecr.begin(); ts_i != tsecr.end(); ts_i++)
+							if (ts_i->second == tse_first)
+								os->add_result(get_id(), ts_i->first, XPROBE_MATCH_YES);
+						tsv_first = timestamps[0];
+						tse_first = timestamps[1];
+					} else {
+						for (ts_i = tsval.begin(); ts_i != tsval.end(); ts_i++)  
+							if (ts_i->second == 2) /* NONE */
+								os->add_result(get_id(), ts_i->first, XPROBE_MATCH_YES);
+						for (ts_i = tsecr.begin(); ts_i != tsecr.end(); ts_i++)
+							if (ts_i->second == 2) /* NONE */
+								os->add_result(get_id(), ts_i->first, XPROBE_MATCH_YES);
+					}
+					if (tg->generate_sig())
+						generate_signature(tg, &sn, request);
+				}
+		} else {
+			if (!sn.timeout() && sn.get_flags() == (TH_ACK|TH_RST)) {
+				/*
+				 * add port to the list of the open ports on target
+				 */
+				tg->add_port(IPPROTO_TCP, request->get_dstport(), XPROBE_TARGETP_CLOSED);
 			}
+			xprobe_debug(XPROBE_DEBUG_MODULES, "[%s] Did not receive SYN ACK packet. %d\n", get_name(), sn.get_dstport());
+		}
 	}
 	return OK;
 }
@@ -259,7 +304,10 @@ int TCP_Handshake_Mod::parse_options(char *tcp_options, int len) {
 				// we are guaranteed to have 8 bytes of option data at tcp_options+lenparsed
 				memcpy(&timestamps[0], tcp_options+lenparsed+2, 4);
 				memcpy(&timestamps[1], tcp_options+lenparsed+6, 4);
-				xprobe_debug(XPROBE_DEBUG_MODULES, "[%s] Timestamps: %d %d\n", get_name(), ntohl(timestamps[0]),ntohl(timestamps[1]));
+				timestamps[0] = ntohl(timestamps[0]);
+				timestamps[1] = ntohl(timestamps[1]);
+
+				xprobe_debug(XPROBE_DEBUG_MODULES, "[%s] Timestamps: %d %d\n", get_name(), timestamps[0],timestamps[1]);
 				if (k < sizeof(opt_order))
 					opt_order[k++]='T';
 				got_timestamp = true;
@@ -302,11 +350,10 @@ int TCP_Handshake_Mod::parse_keyword(int os_id, const char *kwd, const char *val
 	unsigned int vl;
 	map<string, Xprobe_Module_Param_TCP *>::iterator m_i;
 	
-	xprobe_debug(XPROBE_DEBUG_MODULES, "Parsing for %i : %s  = %s\n", os_id,  kwd, val);
+	xprobe_debug(XPROBE_DEBUG_SIGNATURES, "Parsing for %i : %s  = %s\n", os_id,  kwd, val);
     
-	for (m_i = kwd_chk.begin(); m_i != kwd_chk.end(); m_i++){
-		if (m_i->first == kwd)
-			return (m_i->second->parse_param(os_id, val));
+	if ((m_i=kwd_chk.find(kwd)) != kwd_chk.end()) {
+		return (m_i->second->parse_param(os_id, val));
 	}
 	/* parse "tcp_syn_ack_options_order" keyword
 	 * following values are accepted:
@@ -398,6 +445,7 @@ int TCP_Handshake_Mod::get_tcpopts_pack(Target *tg, TCP *tcp) {
 	tcp->set_tos(0x10);
 	tcp->set_fragoff(IP_DF);
 	tcp->set_seq(rand());
+	tcp->set_ack(0);
 	to.one_word = 1460;
 	tcp->set_tcpopt(TCPOPT_MAXSEG, TCPOLEN_MAXSEG, to);
 	memset(&to, 0, sizeof(to.unknown));
