@@ -1,4 +1,4 @@
-/* $Id: static_modules.h,v 1.5 2004/06/09 12:08:24 mederchik Exp $ */
+/* $Id: static_modules.h,v 1.6 2005/02/08 19:39:02 mederchik Exp $ */
 /*
 ** Copyright (C) 2001 Fyodor Yarochkin <fygrave@tigerteam.net>,
 **                    Ofir Arkin       <ofir@sys-security.com>
@@ -33,8 +33,9 @@ extern int icmp_timestamp_mod_init(Xprobe_Module_Hdlr *, char *);
 extern int icmp_inforeq_mod_init(Xprobe_Module_Hdlr *, char *);
 extern int icmp_addrmask_mod_init(Xprobe_Module_Hdlr *, char *);
 extern int icmp_port_unreach_init(Xprobe_Module_Hdlr *, char *);
-extern int tcp_handshake_mod_init(Xprobe_Module_Hdlr *pt, char *);
-extern int portscan_mod_init(Xprobe_Module_Hdlr *pt, char *);
+extern int tcp_handshake_mod_init(Xprobe_Module_Hdlr *, char *);
+extern int portscan_mod_init(Xprobe_Module_Hdlr *, char *);
+extern int tcp_rst_mod_init(Xprobe_Module_Hdlr *, char *);
 
 
 typedef struct xprobe_module_func {
@@ -54,6 +55,7 @@ xprobe_module_func_t mod_init_funcs[]= {
 	{"fingerprint:icmp_info", icmp_inforeq_mod_init },
 	{"fingerprint:icmp_port_unreach", icmp_port_unreach_init},
 	{"fingerprint:tcp_hshake", tcp_handshake_mod_init},
+	{"fingerprint:tcp_rst", tcp_rst_mod_init},
 	{NULL, NULL}
 };
 
