@@ -1,4 +1,4 @@
-/* $Id: xprobe_module_param.cc,v 1.9 2003/08/05 03:35:11 mederchik Exp $ */
+/* $Id: xprobe_module_param.cc,v 1.11 2005/02/08 20:00:36 mederchik Exp $ */
 /*
 ** Copyright (C) 2001 Fyodor Yarochkin <fygrave@tigerteam.net>,
 **                    Ofir Arkin       <ofir@sys-security.com>
@@ -22,6 +22,7 @@
 #include "xprobe.h"
 #include "xprobe_module_param.h"
 #include "interface.h"
+#include "xplib/xplib.h"
 
 extern Interface *ui;
 
@@ -35,6 +36,9 @@ int Xprobe_Module_Param::sig_insert(int os_id, xprobe_module_param_t p) {
 
 int Xprobe_Module_Param::add_param(int param, int orig, OS_Matrix *os) {
     map <int, xprobe_module_param_t>::iterator sig_i;
+	unsigned int ix;
+	long id_diff;
+	vector <int> *vec_ptr;
 
     for (sig_i = osid_sig.begin(); sig_i != osid_sig.end(); sig_i++) {
     
@@ -94,9 +98,31 @@ int Xprobe_Module_Param::add_param(int param, int orig, OS_Matrix *os) {
                         os->add_result(get_id(), (*sig_i).first, XPROBE_MATCH_NO);
 					break;
 	
+				case XPROBE_MODULE_PARAM_INTLIST:
+					vec_ptr = &((*sig_i).second.val_list);
+					for (ix = 0; ix < vec_ptr->size(); ix++) {						
+						if (param == (*vec_ptr)[ix]) {
+							os->add_result(get_id(), (*sig_i).first, XPROBE_MATCH_YES);
+							break;
+						} else {
+							os->add_result(get_id(), (*sig_i).first, XPROBE_MATCH_NO);
+						}
+
+					}
+					break;
+
+				case XPROBE_MODULE_PARAM_STRATEGY:
+					id_diff = param - orig;
+					if ((id_diff > XMP_STRATEGY_THRESHOLD || id_diff < 0)  && (*sig_i).second.low == XMP_STRATEGY_RANDOM)
+						os->add_result(get_id(), (*sig_i).first, XPROBE_MATCH_YES);
+					else if (id_diff > 0 && id_diff <= XMP_STRATEGY_THRESHOLD && (*sig_i).second.low == XMP_STRATEGY_INCREMENTAL)
+						os->add_result(get_id(), (*sig_i).first, XPROBE_MATCH_YES);
+					else if (id_diff == 0 && (*sig_i).second.low == XMP_STRATEGY_ZERO) 
+						os->add_result(get_id(), (*sig_i).first, XPROBE_MATCH_YES);
+					break;
                default:
                /* oops */
-               ui->msg("Something fucked in add_param\n");
+					ui->msg("Something fucked in add_param\n");
                return FAIL;
             } /* switch */
     } /* for */
@@ -155,24 +181,49 @@ int Xprobe_Module_Param::parse_param(int os_id, const char *param) {
 				}
 			}
 			sig_insert(os_id, p);
+			return OK;
 			break;
 
         case XPROBE_MODULE_PARAM_INT:
         case XPROBE_MODULE_PARAM_RANGE:
             if (param[0] == '<') {
                 p.low = 0; p.high = atoi(param+1);
-                sig_insert(os_id, p);
             } else if (param[0] == '>') {
                 p.low = atoi(param+1); p.high = 256;
-                sig_insert(os_id, p);
             } else  {
                 p.low = atoi(param) - 1; p.high = atoi(param) + 1;
-                sig_insert(os_id, p);
             }
+			sig_insert(os_id, p);
             return OK;
             break;
+		case XPROBE_MODULE_PARAM_INTLIST:
+			if (xp_lib::tokenize(param, ',', &p.val_list) == FAIL) {
+				ui->error("xp_lib::tokenize() failed!\n");
+				return FAIL;
+			}
+			sig_insert(os_id, p);
+			return OK;
+			break;
+		case XPROBE_MODULE_PARAM_STRATEGY:
+			switch(param[0]) {
+				case 'R':
+					p.low = XMP_STRATEGY_RANDOM;
+					break;
+				case 'I':
+					p.low = XMP_STRATEGY_INCREMENTAL;
+					break;
+				case '0':
+					p.low = XMP_STRATEGY_ZERO;
+					break;
+				default:
+					ui->msg("xprobe_param module:  unknown value %s\n", param);
+					return FAIL;
+			}
+			sig_insert(os_id, p);
+			return OK;
+			break;
         default:
-            ui->msg("oops.. something fucked up!\n");
+            ui->msg("Xprobe_Module_Param::parse_param(): oops.. something fucked up!\n");
     }
     return FAIL;
 }
@@ -181,6 +232,8 @@ int Xprobe_Module_Param::gen_match(int cnt, OS_Matrix *os) {
 	map <int, xprobe_module_param_t>::iterator sig_i;
 	int i;
 
+	if (cnt < 0)
+		return OK;
 	for (sig_i = osid_sig.begin(); sig_i != osid_sig.end(); sig_i++) 
 		for (i=0; i < cnt; i++)
 			os->add_result(get_id(), sig_i->first, XPROBE_MATCH_YES);
